/*
Copyright 2012 Daniel Morton. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are
permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of
      conditions and the following disclaimer.

   2. Redistributions in binary form must reproduce the above copyright notice, this list
      of conditions and the following disclaimer in the documentation and/or other materials
      provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Daniel Morton ''AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Daniel Morton OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those of the
authors and should not be interpreted as representing official policies, either expressed
or implied, of Daniel Morton.
*/
package ca.quadrilateral.blog.embeddedderbyintomcat.service;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.logging.Logger;

import ca.quadrilateral.blog.embeddedderbyintomcat.data.Product;

public class DataService implements IDataService {
    private static final Logger log = Logger.getLogger(DataService.class.getName());

    @Override
    public Collection<Product> getProducts() {
        final Collection<Product> products = new ArrayList<Product>();
        Connection con = null;
        Statement statement = null;
        ResultSet resultSet = null;
        try {
            con = DriverManager.getConnection("jdbc:derby:databaseName");
            statement = con.createStatement();
            statement.execute("SELECT name, manufacturer, number_in_stock, cost FROM product");
            resultSet = statement.getResultSet();

            while(resultSet.next()) {
                products.add(createProductFromResultSet(resultSet));
            }
        } catch (SQLException e) {
            log.severe("An error occurred loading products from the database: " + e.getMessage());
            throw new RuntimeException("An error occured loading the products from the database");
        } finally {
            if (resultSet != null) {
                try {
                    resultSet.close();
                } catch (SQLException sqle) {
                    log.warning("Unexpected SQLException closing ResultSet resource: " + sqle.getMessage());
                }
            }
            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException sqle) {
                    log.warning("Unexpected SQLException closing Statement resource: " + sqle.getMessage());
                }
            }
            if (con != null) {
                try {
                    con.close();
                } catch (SQLException sqle) {
                    log.warning("Unexpected SQLException closing Connection resource: " + sqle.getMessage());
                }
            }
        }

        return products;
    }

    private Product createProductFromResultSet(ResultSet resultSet) throws SQLException {
        final Product product = new Product();
        product.setName(resultSet.getString(1));
        product.setManufacturer(resultSet.getString(2));
        product.setNumberInStock(resultSet.getInt(3));
        product.setRetailCost(resultSet.getBigDecimal(4));
        return product;
    }
}
